package reachability;

import java.util.Collection;
import java.util.Collections;
import java.util.Vector;

public class Configuration {
    private final int state;
    private final Vector<Integer> vector = new Vector<Integer>();
    
    /**
     * Constructor of class Configuration.
     */
    public Configuration(int state, Vector<Integer> vector) {
        this.state = state;
        this.vector.addAll(vector);
    }
    
    /**
     * Returns the state of this configuration.
     * 
     * @return the state of this configuration
     */
    public int getState() {
        return state;
    }
    
    /**
     * Returns the vector of this configuration.
     * 
     * @return the vector of this configuration
     */
    public Vector<Integer> getVector() {
        return vector;
    }
    
    /**
     * Computes if this configuration is less or equal than the configuration 2.
     * 
     * @param configuration2 the configuration 2
     * @return true if this configuration is less or equal than the configuration 2, false otherwise
     */
    public boolean isLessOrEqual(Configuration configuration2) {
        if (state == configuration2.getState()) {
            for (int i = 0; i < vector.size(); i++) {
                if (vector.get(i) > configuration2.getVector().get(i)) {
                    return false;
                }
            }
        } else {
            return false;
        }
        return true;
    }
    
    /**
     * Computes if this configuration is less than the configuration 2.
     * 
     * @param configuration2 the configuration 2
     * @return true if this configuration is less than the configuration 2, false otherwise
     */
    public boolean isLess(Configuration configuration2) {
        int auxiliaryVariable = 0;
        if (state == configuration2.getState()) {
            for (int i = 0; i < vector.size(); i++) {
                if (vector.get(i) > configuration2.getVector().get(i)) {
                    return false;
                }
                if (vector.get(i) == configuration2.getVector().get(i)) {
                    auxiliaryVariable++;
                }
            }
            if (auxiliaryVariable == vector.size()) {
                return false;
            }
        } else {
            return false;
        }
        return true;
    }
    
    /**
     * Returns the string containing the state and the vector of this configuration.
     * 
     * @return the string
     */
    @Override
    public String toString() {
        return state + vector.toString();
    }
    
    /**
     * Computes if this configuration equals the configuration 2.
     * 
     * @param configuration2 the configuration 2
     * @return true if this configuration equals the configuration 2, false otherwise
     */
    public boolean equals(Configuration configuration2) {
        if (state == configuration2.getState()) {
            for (int i = 0; i < vector.size(); i++) {
                if (vector.get(i) != configuration2.getVector().get(i)) {
                    return false;
                }
            }
        } else {
            return false;
        }
        return true;
    }
}