package reachability;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

public class VASS {
    private final int dimension;
    private final ArrayList<Integer> states = new ArrayList<Integer>();
    private final ArrayList<Rule> rules = new ArrayList<Rule>();

    /**
     * Constructor of class VASS.
     */
    public VASS(int dimension, ArrayList<Integer> states, ArrayList<Rule> rules) {
        this.dimension = dimension;
        this.states.addAll(states);
        this.rules.addAll(rules);
    }
    
    /**
     * Returns the dimension of this VASS.
     * 
     * @return the dimension of this VASS
     */
    public int getDimension() {
        return dimension;
    }
    
    /**
     * Returns the states of this VASS.
     * 
     * @return the states of this VASS
     */
    public List<Integer> getStates() {
        return Collections.unmodifiableList(states);
    }
    
    /**
     * Returns the rules of this VASS.
     * 
     * @return the rules of this VASS
     */
    public List<Rule> getRules() {
        return Collections.unmodifiableList(rules);
    }
    
    /**
     * Computes if the configuration 2 is coverable from the configuration 1.
     * 
     * @param configuration1 the configuration 1
     * @param configuration2 the configuration 2
     * @return true if the configuration 2 is coverable from the configuration 1, false otherwise
     */
    public boolean coverability1(Configuration configuration1, Configuration configuration2) {
        int i;
        int factorialOfDimension = 1;
        for (i = 2; i <= dimension; i++) {
            factorialOfDimension *= i;
        }
        long length = (long) (Math.pow(states.size(), 2)+Math.pow(2*states.size()+1, dimension)*(Math.pow(dimension, 2)*factorialOfDimension*Math.pow(states.size(), 2)+2*Math.pow(dimension, 3)*Math.pow(factorialOfDimension, 2)*Math.pow(states.size(), 4)*Math.pow((2*states.size()+1), dimension)));
        ArrayList<ArrayList<Configuration>> minU = new ArrayList<ArrayList<Configuration>>();
        int j;
        int k;
        int l;
        Vector<Integer> vector = new Vector<Integer>();
        Configuration configuration3;
        int auxiliaryVariable;
        int auxiliaryVariable2;
        
        if (configuration2.isLessOrEqual(configuration1)) {
            return true;
        }
        i = 0;
        minU.add(i, new ArrayList<Configuration>());
        minU.get(i).add(configuration2);
        System.out.println("minU("+(i)+"): {"+minU.get(i).get(0)+"}");
        do {
            minU.add(i+1, new ArrayList<Configuration>());
            minU.get(i+1).addAll(minU.get(i));
            for (j = 0; j < minU.get(i).size(); j++) {
                for (k = 0; k < rules.size(); k++) {
                    if (minU.get(i).get(j).getState() == rules.get(k).getState2()) {
                        vector.clear();
                        for (l = 0; l < dimension; l++) {
                            if (minU.get(i).get(j).getVector().get(l) == 1 && rules.get(k).getVector().get(l) == 1) {
                                vector.add(l, 1);
                            } else {
                                vector.add(l, minU.get(i).get(j).getVector().get(l) - rules.get(k).getVector().get(l));
                            }
                        }
                        configuration3 = new Configuration(rules.get(k).getState1(), new Vector<Integer>(vector));
                        if (configuration3.isLessOrEqual(configuration1)) {
                            return true;
                        }
                        auxiliaryVariable = 0;
                        l = 0;
                        do {
                            auxiliaryVariable2 = -1;
                            if (configuration3.isLess(minU.get(i+1).get(l))) {
                                auxiliaryVariable2 = l;
                            } else {
                                l++;
                            }
                            if (auxiliaryVariable2 != -1) {
                                minU.get(i+1).remove(auxiliaryVariable2);
                            }
                            if (l == minU.get(i+1).size()) {
                                auxiliaryVariable = 1;
                            }
                        } while (auxiliaryVariable == 0);
                        auxiliaryVariable = 0;
                        for (l = 0; l < minU.get(i+1).size(); l++) {
                            if (minU.get(i+1).get(l).isLessOrEqual(configuration3)) {
                                auxiliaryVariable = 1;
                            }
                        }
                        if (auxiliaryVariable == 0) {
                            minU.get(i+1).add(configuration3);
                        }
                    }
                }
            }
            System.out.print("minU("+(i+1)+"): {");
            for (j = 0; j < minU.get(i+1).size(); j++) {
                System.out.print(minU.get(i+1).get(j));
                if (j < minU.get(i+1).size()-1) {
                    System.out.print(", ");
                }
            }
            System.out.println("}");
            i++;
            auxiliaryVariable = 0;
            if (minU.get(i-1).size() == minU.get(i).size()) {
                for (j = 0; j < minU.get(i-1).size(); j++) {
                    for (k = 0; k < minU.get(i).size(); k++) {
                        if (minU.get(i-1).get(j).getState() == minU.get(i).get(k).getState() && minU.get(i-1).get(j).getVector().equals(minU.get(i).get(k).getVector())) {
                            auxiliaryVariable++;
                        }
                    }
                }
            }
        } while (minU.get(i-1).size() != auxiliaryVariable && i <= length);
        return false;
    }
    
    /**
     * Computes if the configuration 2 is coverable from the configuration 1.
     * 
     * @param configuration1 the configuration 1
     * @param configuration2 the configuration 2
     * @return true if the configuration 2 is coverable from the configuration 1, false otherwise
     */
    public boolean coverability2(Configuration configuration1, Configuration configuration2) {
        int i;
        ArrayList<ArrayList<Configuration>> minU = new ArrayList<ArrayList<Configuration>>();
        int j;
        int k;
        int l;
        Vector<Integer> vector = new Vector<Integer>();
        Configuration configuration3;
        int auxiliaryVariable;
        int auxiliaryVariable2;
        
        if (configuration2.isLessOrEqual(configuration1)) {
            return true;
        }
        i = 0;
        minU.add(i, new ArrayList<Configuration>());
        minU.get(i).add(configuration2);
        System.out.println("minU("+(i)+"): {"+minU.get(i).get(0)+"}");
        do {
            minU.add(i+1, new ArrayList<Configuration>());
            minU.get(i+1).addAll(minU.get(i));
            for (j = 0; j < minU.get(i).size(); j++) {
                for (k = 0; k < rules.size(); k++) {
                    if (minU.get(i).get(j).getState() == rules.get(k).getState2()) {
                        vector.clear();
                        for (l = 0; l < dimension; l++) {
                            if (minU.get(i).get(j).getVector().get(l) == 1 && rules.get(k).getVector().get(l) == 1) {
                                vector.add(l, 1);
                            } else {
                                vector.add(l, minU.get(i).get(j).getVector().get(l) - rules.get(k).getVector().get(l));
                            }
                        }
                        configuration3 = new Configuration(rules.get(k).getState1(), new Vector<Integer>(vector));
                        if (configuration3.isLessOrEqual(configuration1)) {
                            return true;
                        }
                        auxiliaryVariable = 0;
                        l = 0;
                        do {
                            auxiliaryVariable2 = -1;
                            if (configuration3.isLess(minU.get(i+1).get(l))) {
                                auxiliaryVariable2 = l;
                            } else {
                                l++;
                            }
                            if (auxiliaryVariable2 != -1) {
                                minU.get(i+1).remove(auxiliaryVariable2);
                            }
                            if (l == minU.get(i+1).size()) {
                                auxiliaryVariable = 1;
                            }
                        } while (auxiliaryVariable == 0);
                        auxiliaryVariable = 0;
                        for (l = 0; l < minU.get(i+1).size(); l++) {
                            if (minU.get(i+1).get(l).isLessOrEqual(configuration3)) {
                                auxiliaryVariable = 1;
                            }
                        }
                        if (auxiliaryVariable == 0) {
                            minU.get(i+1).add(configuration3);
                        }
                    }
                }
            }
            System.out.print("minU("+(i+1)+"): {");
            for (j = 0; j < minU.get(i+1).size(); j++) {
                System.out.print(minU.get(i+1).get(j));
                if (j < minU.get(i+1).size()-1) {
                    System.out.print(", ");
                }
            }
            System.out.println("}");
            i++;
            auxiliaryVariable = 0;
            if (minU.get(i-1).size() == minU.get(i).size()) {
                for (j = 0; j < minU.get(i-1).size(); j++) {
                    for (k = 0; k < minU.get(i).size(); k++) {
                        if (minU.get(i-1).get(j).getState() == minU.get(i).get(k).getState() && minU.get(i-1).get(j).getVector().equals(minU.get(i).get(k).getVector())) {
                            auxiliaryVariable++;
                        }
                    }
                }
            }
        } while (minU.get(i-1).size() != auxiliaryVariable);
        return false;
    }
}